# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-04 10:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0006_subscription_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='subscription',
            name='parent',
        ),
        migrations.AddField(
            model_name='subscription',
            name='field',
            field=models.CharField(blank=True, help_text='if set to a field name on the object model, the field will be passed instead of the object instance', max_length=255, null=True),
        ),
    ]
